//#version 330 core

attribute vec4 in_position;
attribute vec2 texCoord;
attribute vec3 nvec;
attribute vec3 Tng;
uniform mat4 model_matrix;
uniform mat4 view_matrix;
uniform mat4 projection_matrix;
uniform mat4 light_transformation_matrix;
varying vec2 texel;
varying vec4 SM_tex_coord;
uniform int useNMvrtx; //use normal mapping
uniform int usePMvrtx; //use parallax normal mapping
varying vec3 normal;
varying vec3 lightVec;
varying vec3 halfVec;
varying vec3 eyeVec;

varying vec3 eyeVec4prlx; // for parallax

void main(void)
{
	texel=vec2(texCoord);
	gl_Position =   projection_matrix * view_matrix * model_matrix * in_position;
	SM_tex_coord = light_transformation_matrix * model_matrix * in_position;

	normal = normalize (view_matrix * model_matrix * vec4(nvec,0.0));
	
	vec3 vVertex = vec3(view_matrix * model_matrix * in_position);
	//vec3 LightPos=vec3(view_matrix * vec4(25,0.5,-2.7,0.0));
	vec3 LightPos=vec3(view_matrix * vec4(24,5.0,-2.7,0.0));

	lightVec = normalize(LightPos - vVertex);
	eyeVec = normalize (vVertex);
	halfVec = normalize(vVertex + lightVec) ; 

	if (useNMvrtx == 1 || usePMvrtx == 1)
	{
		vec3 tangent = normalize (view_matrix * model_matrix * vec4(Tng,0.0));
		vec3 binormal = cross (normal, tangent);

		// TngConvertmx : matrix that converts to tangent space
		mat3 TngConvertmx=mat3(tangent.x,binormal.x,normal.x,tangent.y,binormal.y,normal.y,tangent.z,binormal.z,normal.z);

		// parallax mapping and normal mapping can be both applied, or only one of them
	
		vec3 v;
		if(usePMvrtx == 1) // just compute view vector in tangent space and send to fragment shader in a specific variable
		{
			v=eyeVec;
			eyeVec4prlx = normalize(TngConvertmx*v);
		}


		if(useNMvrtx == 1) // convert all vectors relevant to lighting to tangent space and send to fragment shader
		{
			v=eyeVec;
			eyeVec = normalize(TngConvertmx*v);

			vec3 v=lightVec;
			lightVec = normalize(TngConvertmx*v);
		
			v = halfVec;
			halfVec = normalize(TngConvertmx*v) ; 
		}

	}
	
}